/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


#include <libc.h>
#include <string.h>
#include <stdio.h>

#include "OO7.hxx"
#include "GenParams.h"
#include "VarParams.h"

/////////////////////////////////////////////////////////////////
//
// Traversal #7.  Pick a random atomic part; traverse from the 
// part to its containing composite part P.  From P, traverse
// backwards up to all base assemblies referencing P; for each
// such base assembly, traverse upward through the assembly
// hierarchy until the root is reached.  This traversal coupled
// with traversal #1 ensures that all pointers are bi-directional.
//
/////////////////////////////////////////////////////////////////

extern int	debugMode;


//////////////////////////////////////////////////////////////////
//
// Routine to initiate Traversal #7
//
//////////////////////////////////////////////////////////////////


int traverse7()
{
    // randomly select an atomic part and traverse upwards from there

    // set random seed so "hot" runs are truly hot
    srandom(1);

    int partId = (int) (random() % TotalAtomicParts) + 1;
    AtomicPart* atom = AtomicPart::extent_id->getFirst(partId) ;

    if (debugMode) {
        printf("  In Traverse7, starting at atomic part %d\n", partId);
        printf("  In Traverse7, traversing to composite part %d\n",
		atom->partOf->id);
    }

    return atom->partOf->traverse7();
}



//////////////////////////////////////////////////////////////////
//
// CompositePart Method for Traversal #7
//
//////////////////////////////////////////////////////////////////


int CompositePart::traverse7()
{
    if (debugMode) {
        printf("\tCompositePart::traverse7(id = %d)\n", id);
    }

    // search up the design hierarchy (along the private path)

    PartIdSet visitedBaseIds;
    PartIdSet visitedComplexIds;
    BaseAssembly* base;
    int count = 0;
    Iterator<BaseAssembly> iter(usedInPriv) ;
    for (; iter.more(); iter.next()) {
      base = iter.get() ;
      if (debugMode) {
	printf("\tConsidering base assembly %d\n", base->id);
      }
      if (!visitedBaseIds.contains(base->id)) {
	visitedBaseIds.insert(base->id);
	count += base->traverse7(visitedComplexIds);
      }
    }
    return count;
}


//////////////////////////////////////////////////////////////////
//
// BaseAssembly Method for Traversal #7
//
//////////////////////////////////////////////////////////////////


int BaseAssembly::traverse7(PartIdSet& visitedComplexIds)
{
    if (debugMode) {
        printf("\t\tBaseAssembly::traverse7(id = %d)\n", id);
    }

    // visit this assembly and move on up the design hierarchy

    int count = 1;

    DoNothing();

    if (!visitedComplexIds.contains(superAssembly->id)) {
        count += superAssembly->traverse7(visitedComplexIds);
    }
    return count;
}


//////////////////////////////////////////////////////////////////
//
// ComplexAssembly Method for Traversal #7
//
//////////////////////////////////////////////////////////////////


int ComplexAssembly::traverse7(PartIdSet& visitedComplexIds)
{
    if (debugMode) {
        printf("\t\t\tComplexAssembly::traverse7(id = %d)\n", id);
    }

    // process this assembly and move on up the design hierarchy

    int count = 1;

    visitedComplexIds.insert(id);

    DoNothing(); 

    if (superAssembly != NULL) {
      if (!visitedComplexIds.contains(superAssembly->id)) {
	count += superAssembly->traverse7(visitedComplexIds);
      }
    }
    return count;
}

